#ifndef SCRIPT_LOADED
#define SCRIPT_LOADED

#include <stdio.h>

#include <iostream.h>
#include <fstream.h>
#include <strstream.h>
#include <assert.h>
#include <vector.h>
#include <strings.h>

typedef vector <float> scan;

#include "Timer.h"

const int maxStringLength = 4096;
const int maxNumberOfProximityReadings = 1024;


const int newScript = 0;
const int oldScript = 1;

#define origScriptOpenMark "@OPEN"
#define origScriptPosMark "#ROBOT"
#define origScriptBumperMark "#BUMP"
#define origScriptSonarMark "#SONAR"
#define origScriptLaserMark "#LASER"
#define origScriptMarkerMark  "#MARKER:"
#define origScriptTimeMark "@SENS"
#define origScriptImageMark "#IMAGE"
const int origScriptTimeOffset = 9;

//#define newScriptOpenMark "begin(patternset)"
#define newScriptOpenMark "begin(pattern)"
#define newScriptPosMark "position:"
#define newScriptBumperMark ""
#define newScriptSonarMark "sonars"
#define newScriptLaserMark "lasers"
#define newScriptMarkerMark  "marker:"
#define newScriptTimeMark "time:"
#define newScriptImageMark "image"
const int newScriptTimeOffset = 0;

#define scriptBumpMark "#BUMP"


typedef struct
{
  double x;
  double y;
  double rot;

  bool isNew;
} position;
  

class proximityScan {
public:
  explicit proximityScan(int numberOfReadings);

  scan  reading;
  
  inline bool isNew();
  inline void setNew();
  inline void setOld();
  inline bool getReadings(char *line);
  inline bool writeReadings();
private:
  bool _isNew;
};



typedef struct{
  long secs;
  double usecs;
} scriptTimeold;


#define SCRIPT_TYPE_NOT_SET 0
#define OLD_SCRIPT_TYPE     1
#define NEW_SCRIPT_TYPE     2

/////////////////////////////////////////////////////////////////
//  Reads a script and sends the data via tcx.
/////////////////////////////////////////////////////////////////

class script {
public:
  
  explicit script( string& fileName,
		   const char *refFileName = NULL,
		   float timeFactor = 1.0,
		   bool skipIfRobotStands = false,
		   int startTime = 0,
		   int numberOfLaserScanners = 1,
		   int scriptType = SCRIPT_TYPE_NOT_SET);

  //  void addNoise( MotionNoiseParameter& noiseParams);
  inline void setTimeFactor( float timeFactor);

  bool getNextItem( bool considerTime = true);
  bool getNextLine( char* line);
  void sendData();
  
  inline proximityScan  currentLaserScan();
  inline proximityScan  currentSonarScan();
  inline position       currentRobotPosition();
  inline float          elapsedTime();

  //accessors
  proximityScan getSonar(){return sonar;}
  proximityScan getLaser(){return laser;}

  position getRefPosition(){return refPosition;}

private:
  char *fileName;
  proximityScan laser;
  proximityScan sonar;
  position scriptPosition;
  char *timeMark;
  char *positionMark;
  char *laserMark;
  char *sonarMark;
  char *imageMark;
  int timeOffset;
  int _scriptType;
  
  Timer _currentTime;
  Timer _startTime;

#ifdef USE_ZLIB
  gzFile _zFile;
#else
  FILE*  _file;
#endif
  FILE*  _stdin;

  // file with reference positions
  FILE*  _refFile;
  // reference position - i.e. real robot position
  position refPosition;
  // init refPosisition
  void initRefPosition();
  
  Timer  _realTime;
  Timer  _elapsedTime;

  float  _timeFactor;
  float  _timeSkipped;
  
  bool   _skipIfRobotStands;
  
  inline bool readingLine(char* line, char* mark);
};

#endif
